@@ -1,8 +1,8 @@
 use super::block::{Block, DualBlock};
 use crate::quadratic::{QuadraticSurd, QuadraticSurdBase};
 use crate::traits::{Approximation, Computable, WithSigned, WithUnsigned};
-use core::str::FromStr;
 use core::convert::TryFrom;
+use core::str::FromStr;
 use num_integer::Integer;
 use num_rational::Ratio;
 use num_traits::{CheckedAdd, CheckedMul, Num, NumRef, One, RefNum, Signed, Zero};
@@ -672,7 +672,7 @@ impl<T> Add<Ratio<T>> for ContinuedFraction<T> {
     type Output = Self;
 
     fn add(self, rhs: Ratio<T>) -> Self {
-        unimplemented!() // TODO: implement by converting to ratio or quadratic surd
+        unimplemented!() // TODO (v0.1): implement by converting to ratio or quadratic surd
     }
 }
 
