@@ -1,7 +1,8 @@
 use crate::cont_frac::ContinuedFraction;
 use crate::traits::{Approximation, Computable, FromSqrt, FromSqrtError, WithSigned, WithUnsigned};
-use core::ops::{Add, AddAssign, Div, Mul, Neg, Sub};
+#[cfg(feature = "complex")]
 use core::convert::TryFrom;
+use core::ops::{Add, AddAssign, Div, Mul, Neg, Sub};
 use num_integer::{sqrt, Integer, Roots};
 use num_traits::{
     CheckedAdd, CheckedMul, FromPrimitive, NumRef, One, RefNum, Signed, ToPrimitive, Zero,
@@ -17,7 +18,11 @@ impl<T: Integer + NumRef + Clone + Roots + Signed> QuadraticSurdBase for T {}
 /// A quadratic number represented as `(a + b*sqrt(r)) / c`.
 /// If the support for complex number is enabled, then this struct can represent
 /// any quadratic integers.
-#[derive(PartialEq, Eq, Hash, Clone, Debug, Copy)] // TODO (v0.0.4): implement PartialEq/Eq with binop reduction
+///
+/// Note that [Eq] is not implemented for quadratic surd, as the root is not ensured
+/// to be reduced to the square free form, thus it's possible that two surds
+/// representing the same value have different coefficients.
+#[derive(Hash, Clone, Debug, Copy)]
 pub struct QuadraticSurd<T> {
     a: T,
     b: T, // zero when reduced if the surd is a rational number
@@ -33,17 +38,26 @@ impl<T> QuadraticSurd<T> {
 }
 
 impl<T: Integer> QuadraticSurd<T> {
+    /// Determine if the surd is an integer
     #[inline]
     pub fn is_integer(&self) -> bool {
         self.r >= T::zero() && self.c.is_one() && self.b.is_zero()
     }
 
+    /// Determine if the surd is a rational number
     #[inline]
     pub fn is_rational(&self) -> bool {
         self.b.is_zero() || self.r.is_zero()
     }
 
-    /// Determine if the quadratic number has no real part (i.e. a = 0, b != 0)
+    /// Determine if the surd is a complex number
+    #[inline]
+    #[cfg(feature = "complex")]
+    pub fn is_complex(&self) -> bool {
+        self.r < T::zero()
+    }
+
+    /// Determine if the quadratic number has no rational part (i.e. a = 0, b != 0)
     #[inline]
     pub fn is_pure(&self) -> bool {
         self.a.is_zero() && !self.b.is_zero()
@@ -92,13 +106,17 @@ impl<T: QuadraticSurdBase> QuadraticSurd<T>
 where
     for<'r> &'r T: RefNum<T>,
 {
-    // TODO (v0.0.4): add from_complex, to_complex, is_complex
-
+    // Simplify the surd into normalized form
     fn reduce(&mut self) {
         if self.c.is_zero() {
             panic!("denominator is zero");
         }
 
+        // ensure b, r are zeros at the same time
+        if self.b.is_zero() {
+            self.r = T::zero();
+        }
+
         // test if the surd is rational
         let root = sqrt(self.r.abs());
         if &root * &root == self.r {
@@ -137,6 +155,7 @@ where
         }
     }
 
+    // Try to reduce the root base with a possible factor.
     fn reduce_root_hinted(self, hint: T) -> Self {
         let hint = hint.abs();
         if hint.is_zero() || hint.is_one() || hint.is_negative() {
@@ -158,7 +177,7 @@ where
     }
 
     /// Create a surd represented as `(a + b*sqrt(r)) / c` where `a`, `b`, `c`, `r` are integers.
-    /// 
+    ///
     /// # Panics
     /// If `c` is zero or `r` is negative when the `complex` feature is not enabled.
     #[inline]
@@ -174,11 +193,11 @@ where
     }
 
     /// Create a surd represented as `a + b sqrt(r)` where a, b, r are rationals.
-    /// 
+    ///
     /// # Panics
     /// If `r` is negative when the `complex` feature is not enabled.
     #[inline]
-    pub fn from_rational(a: Ratio<T>, b: Ratio<T>, r: Ratio<T>) -> Self {
+    pub fn from_rationals(a: Ratio<T>, b: Ratio<T>, r: Ratio<T>) -> Self {
         #[cfg(not(feature = "complex"))]
         if r.is_negative() {
             panic!("Negative root is not supported without the `complex` feature");
@@ -210,14 +229,14 @@ where
         let two = T::one() + T::one();
         let four = &two * &two;
         let delta = &b * &b - Ratio::from(four) * &a * c;
-        
+
         #[cfg(not(feature = "complex"))]
         if delta.is_negative() {
             return None;
         }
 
         let aa = Ratio::from(two) * a;
-        Some(Self::from_rational(-b * aa.recip(), aa.recip(), delta))
+        Some(Self::from_rationals(-b * aa.recip(), aa.recip(), delta))
     }
 
     /// Returns the reciprocal of the surd
@@ -264,7 +283,7 @@ where
     }
 
     /// Round the surd to zero.
-    /// 
+    ///
     /// # Panics
     /// if the number is complex
     pub fn trunc(self) -> Self {
@@ -274,11 +293,7 @@ where
         // TODO: truncate to gaussian integers if complex
         let bneg = self.b.is_negative();
         let br = sqrt(&self.b * &self.b * self.r);
-        let num = if bneg {
-            self.a - br
-        } else {
-            self.a + br
-        };
+        let num = if bneg { self.a - br } else { self.a + br };
         return Self::from(num / self.c);
     }
 
@@ -289,7 +304,7 @@ where
     }
 
     /// Get the fractional part of the surd, ensuring `self.trunc() + self.fract() == self`
-    /// 
+    ///
     /// # Panics
     /// if the number is complex
     #[inline]
@@ -319,7 +334,7 @@ where
     }
 
     /// Converts to an integer, rounding towards zero
-    /// 
+    ///
     /// # Panics
     /// if the number is complex
     #[inline]
@@ -334,7 +349,7 @@ where
     }
 
     /// Converts to a rational, rounding square root towards zero
-    /// 
+    ///
     /// # Panics
     /// if the number is complex
     #[inline]
@@ -352,7 +367,7 @@ where
     }
 
     /// Rounds towards minus infinity
-    /// 
+    ///
     /// # Panics
     /// if the number is complex
     pub fn floor(self) -> Self {
@@ -373,7 +388,7 @@ where
     }
 
     /// `.floor()` with reference
-    /// 
+    ///
     /// # Panics
     /// if the number is complex
     #[inline]
@@ -382,7 +397,7 @@ where
     }
 
     /// Test if the surd number is positive
-    /// 
+    ///
     /// # Panics
     /// if the number is complex
     pub fn is_positive(&self) -> bool {
@@ -407,7 +422,7 @@ where
     }
 
     /// Test if the surd number is negative
-    /// 
+    ///
     /// # Panics
     /// if the number is complex
     pub fn is_negative(&self) -> bool {
@@ -432,6 +447,26 @@ where
     }
 }
 
+impl<T: QuadraticSurdBase> PartialEq for QuadraticSurd<T>
+where
+    for<'r> &'r T: RefNum<T>,
+{
+    fn eq(&self, other: &Self) -> bool {
+        // shortcut if every components are matched
+        if self.a == other.a && self.b == other.b && self.c == other.c && self.r == other.r {
+            return true;
+        }
+        if self.r == other.r || self.r.is_zero() || other.r.is_zero() {
+            return false;
+        }
+
+        match reduce_bin_op(self.clone(), other.clone()) {
+            Some((l, r)) => l.a == r.a && l.b == r.b && l.c == r.c && l.r == r.r,
+            None => false,
+        }
+    }
+}
+
 impl<T> Into<(T, T, T, T)> for QuadraticSurd<T> {
     /// Deconstruct the quadratic surd `(a + b*sqrt(r)) / c` into tuple `(a,b,c,r)`
     fn into(self) -> (T, T, T, T) {
@@ -444,15 +479,15 @@ where
     for<'r> &'r T: RefNum<T>,
 {
     /// Try to eliminate factors of root that is a squared number.
-    /// 
+    ///
     /// It will only try trivial division for several small primes.
     /// For a quadratic surd with large root, consider deconstructing
     /// the surd by `.into()` and then reduce yourself.
     fn reduce_root(&mut self) {
         const SMALL_PRIMES: [u8; 54] = [
-            2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97,
-            101, 103, 107, 109, 113, 127, 131, 137, 139, 149, 151, 157, 163, 167, 173, 179, 181, 191, 193,
-            197, 199, 211, 223, 227, 229, 233, 239, 241, 251,
+            2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83,
+            89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139, 149, 151, 157, 163, 167, 173, 179,
+            181, 191, 193, 197, 199, 211, 223, 227, 229, 233, 239, 241, 251,
         ];
         for p in SMALL_PRIMES {
             let p = T::from_u8(p).unwrap();
@@ -470,7 +505,7 @@ where
             }
         }
     }
-    
+
     /// Returns a reduced version of self.
     ///
     /// This method will try to eliminate common divisors between a, b, c and
@@ -485,10 +520,6 @@ where
     }
 }
 
-fn quadsurd_to_f64<T: Integer + ToPrimitive>(a: T, b: T, c: T, r: T) -> Option<f64> {
-    Some((a.to_f64()? + b.to_f64()? * r.to_f64()?.sqrt()) / c.to_f64()?)
-}
-
 impl<
         T: Integer + Clone + NumRef + CheckedAdd + CheckedMul + WithSigned<Signed = U>,
         U: QuadraticSurdBase,
@@ -625,9 +656,10 @@ where
 
 #[cfg(feature = "complex")]
 impl<T: QuadraticSurdBase + WithUnsigned<Unsigned = U> + AddAssign, U> TryFrom<QuadraticSurd<T>>
-    for ContinuedFraction<U> 
+    for ContinuedFraction<U>
 where
-    for<'r> &'r T: RefNum<T>, {
+    for<'r> &'r T: RefNum<T>,
+{
     type Error = ();
 
     fn try_from(s: QuadraticSurd<T>) -> Result<Self, ()> {
@@ -654,10 +686,12 @@ where
     fn approximated(&self, limit: &T) -> Approximation<Ratio<T>> {
         ContinuedFraction::<U>::from(self.clone()).approximated(limit)
     }
-    
+
     #[cfg(feature = "complex")]
     fn approximated(&self, limit: &T) -> Approximation<Ratio<T>> {
-        ContinuedFraction::<U>::try_from(self.clone()).expect("only real numbers can be approximated by a rational number").approximated(limit)
+        ContinuedFraction::<U>::try_from(self.clone())
+            .expect("only real numbers can be approximated by a rational number")
+            .approximated(limit)
     }
 }
 
@@ -687,16 +721,21 @@ impl<T: Integer + fmt::Display> fmt::Display for QuadraticSurd<T> {
     }
 }
 
-// reduce root base for binary operation
+// Reduce root base for binary operands. Return None if the two bases
+// cannot be matched. This function assumes the root bases on both sides
+// are not zero
 #[inline]
 #[cfg(not(feature = "complex"))]
 fn reduce_bin_op<T: QuadraticSurdBase>(
     lhs: QuadraticSurd<T>,
     rhs: QuadraticSurd<T>,
-) -> (QuadraticSurd<T>, QuadraticSurd<T>)
+) -> Option<(QuadraticSurd<T>, QuadraticSurd<T>)>
 where
     for<'r> &'r T: RefNum<T>,
 {
+    debug_assert!(!lhs.r.is_zero());
+    debug_assert!(!rhs.r.is_zero());
+
     let result = if lhs.r > rhs.r {
         let hint = &lhs.r / &rhs.r;
         (lhs.reduce_root_hinted(hint), rhs)
@@ -707,11 +746,11 @@ where
         (lhs, rhs)
     };
 
-    if result.0.r != result.1.r {
-        panic!("two root bases are not compatible!")
+    if result.0.r == result.1.r {
+        Some(result)
+    } else {
+        None
     }
-
-    result
 }
 
 #[inline]
@@ -719,12 +758,13 @@ where
 fn reduce_bin_op<T: QuadraticSurdBase>(
     lhs: QuadraticSurd<T>,
     rhs: QuadraticSurd<T>,
-) -> (QuadraticSurd<T>, QuadraticSurd<T>)
+) -> Option<(QuadraticSurd<T>, QuadraticSurd<T>)>
 where
     for<'r> &'r T: RefNum<T>,
 {
-    if (lhs.r.is_negative() && rhs.r.is_positive()) || (lhs.r.is_positive() && rhs.r.is_negative()) {
-        panic!("two root bases are not compatible!")
+    if (lhs.r.is_negative() && rhs.r.is_positive()) || (lhs.r.is_positive() && rhs.r.is_negative())
+    {
+        return None;
     }
 
     let result = if lhs.r.abs() > rhs.r.abs() {
@@ -737,11 +777,22 @@ where
         (lhs, rhs)
     };
 
-    if result.0.r != result.1.r {
-        panic!("two root bases are not compatible!")
+    if result.0.r == result.1.r {
+        Some(result)
+    } else {
+        None
     }
+}
 
-    result
+#[inline]
+fn reduce_bin_op_unwrap<T: QuadraticSurdBase>(
+    lhs: QuadraticSurd<T>,
+    rhs: QuadraticSurd<T>,
+) -> (QuadraticSurd<T>, QuadraticSurd<T>)
+where
+    for<'r> &'r T: RefNum<T>,
+{
+    reduce_bin_op(lhs, rhs).expect("two root bases are not compatible!")
 }
 
 macro_rules! arith_impl {
@@ -753,7 +804,22 @@ macro_rules! arith_impl {
         {
             type Output = QuadraticSurd<T>;
             fn $method(self, rhs: QuadraticSurd<T>) -> QuadraticSurd<T> {
-                let (lhs, rhs) = reduce_bin_op(self, rhs);
+                // shortcuts for trivial cases
+                if self.is_integer() {
+                    return rhs.$method(self.a);
+                }
+                if rhs.is_integer() {
+                    return self.$method(rhs.a);
+                }
+                if self.is_rational() {
+                    return rhs.$method(Ratio::<T>::new(self.a, self.c));
+                }
+                if rhs.is_rational() {
+                    return self.$method(Ratio::<T>::new(rhs.a, rhs.c));
+                }
+
+                // ensure that two operands have compatible bases
+                let (lhs, rhs) = reduce_bin_op_unwrap(self, rhs);
 
                 if lhs.c == rhs.c {
                     return QuadraticSurd::new(
@@ -768,13 +834,38 @@ macro_rules! arith_impl {
                 let lhs_r = &lcm / lhs.c;
                 let rhs_r = &lcm / rhs.c;
                 QuadraticSurd::new(
-                    (lhs.a / &lhs_r).$method(rhs.a / &rhs_r),
-                    (lhs.b / &lhs_r).$method(rhs.b / &rhs_r),
+                    (lhs.a * &lhs_r).$method(rhs.a * &rhs_r),
+                    (lhs.b * lhs_r).$method(rhs.b * rhs_r),
                     lcm,
                     rhs.r,
                 )
             }
         }
+
+        impl<T: QuadraticSurdBase> $imp<Ratio<T>> for QuadraticSurd<T>
+        where
+            for<'r> &'r T: RefNum<T>,
+        {
+            type Output = QuadraticSurd<T>;
+
+            #[inline]
+            fn $method(self, rhs: Ratio<T>) -> QuadraticSurd<T> {
+                if &self.c == rhs.denom() {
+                    return QuadraticSurd::new(self.a.$method(rhs.numer()), self.b, self.c, self.r);
+                }
+
+                let lcm = self.c.lcm(&rhs.denom());
+                let lhs_r = &lcm / self.c;
+                let rhs_r = &lcm / rhs.denom();
+                QuadraticSurd::new(
+                    (self.a * lhs_r).$method(rhs.numer() * rhs_r),
+                    self.b,
+                    lcm,
+                    self.r,
+                )
+            }
+        }
+
         // Abstracts the a/b `op` c/1 = (a*1 `op` b*c) / (b*1) = (a `op` b*c) / b pattern
         impl<T: QuadraticSurdBase> $imp<T> for QuadraticSurd<T>
         where
@@ -805,7 +896,26 @@ where
     }
 }
 
-// TODO (v0.1): support ops with Ratio
+impl<T: QuadraticSurdBase> Mul<Ratio<T>> for QuadraticSurd<T>
+where
+    for<'r> &'r T: RefNum<T>,
+{
+    type Output = QuadraticSurd<T>;
+    #[inline]
+    fn mul(self, rhs: Ratio<T>) -> QuadraticSurd<T> {
+        let (rhs_n, rhs_d) = rhs.into();
+        let ab_gcd = self.a.gcd(&self.b).gcd(&rhs_d);
+        let c_gcd = self.c.gcd(&rhs_n);
+        let numer_rem = rhs_n / &c_gcd;
+        let denum_rem = rhs_d / &ab_gcd;
+        QuadraticSurd::new(
+            self.a / &ab_gcd * &numer_rem,
+            self.b / ab_gcd * numer_rem,
+            self.c / c_gcd * denum_rem,
+            self.r,
+        )
+    }
+}
 
 impl<T: QuadraticSurdBase> Mul<QuadraticSurd<T>> for QuadraticSurd<T>
 where
@@ -814,11 +924,25 @@ where
     type Output = QuadraticSurd<T>;
     #[inline]
     fn mul(self, rhs: QuadraticSurd<T>) -> QuadraticSurd<T> {
-        // TODO: shortcuts if rhs is integer, rational
-        if self.is_pure() && rhs.is_pure() {
-            return Self::new(T::zero(), self.b * rhs.b, self.c * rhs.c, self.r * rhs.r)
+        // shortcuts for trivial cases
+        if self.is_integer() {
+            return rhs * self.a;
         }
-        let (lhs, rhs) = reduce_bin_op(self, rhs);
+        if rhs.is_integer() {
+            return self * rhs.a;
+        }
+        if self.is_rational() {
+            return rhs * Ratio::<T>::new(self.a, self.c);
+        }
+        if rhs.is_rational() {
+            return self * Ratio::<T>::new(rhs.a, rhs.c);
+        }
+        if self.is_pure() && rhs.is_pure() {
+            return Self::new(T::zero(), self.b * rhs.b, self.c * rhs.c, self.r * rhs.r);
+        }
+
+        // ensure that two operands have compatible bases
+        let (lhs, rhs) = reduce_bin_op_unwrap(self, rhs);
 
         let gcd_lr = (&lhs.a).gcd(&lhs.b).gcd(&rhs.c); // gcd between lhs numerator and rhs denominator
         let gcd_rl = (&rhs.a).gcd(&rhs.b).gcd(&rhs.c); // gcd between rhs numerator and lhs denominator
@@ -847,6 +971,27 @@ where
     }
 }
 
+impl<T: QuadraticSurdBase> Div<Ratio<T>> for QuadraticSurd<T>
+where
+    for<'r> &'r T: RefNum<T>,
+{
+    type Output = QuadraticSurd<T>;
+    #[inline]
+    fn div(self, rhs: Ratio<T>) -> QuadraticSurd<T> {
+        let (rhs_n, rhs_d) = rhs.into();
+        let ab_gcd = self.a.gcd(&self.b).gcd(&rhs_n);
+        let c_gcd = self.c.gcd(&rhs_d);
+        let denom_rem = rhs_n / &ab_gcd;
+        let numer_rem = rhs_d / &c_gcd;
+        QuadraticSurd::new(
+            self.a / &ab_gcd * &numer_rem,
+            self.b / ab_gcd * numer_rem,
+            self.c / &c_gcd * denom_rem,
+            self.r,
+        )
+    }
+}
+
 impl<T: QuadraticSurdBase> Div<QuadraticSurd<T>> for QuadraticSurd<T>
 where
     for<'r> &'r T: RefNum<T>,
@@ -960,12 +1105,36 @@ where
         if self.r < T::zero() {
             return None;
         }
-        quadsurd_to_f64(
-            self.a.to_i64()?,
-            self.b.to_i64()?,
-            self.c.to_i64()?,
-            self.r.to_i64()?,
-        )
+
+        Some((self.a.to_f64()? + self.b.to_f64()? * self.r.to_f64()?.sqrt()) / self.c.to_f64()?)
+    }
+}
+
+#[cfg(feature = "num-complex")]
+mod complex {
+    use super::*;
+    use num_complex::{Complex32, Complex64};
+
+    impl<T: QuadraticSurdBase + ToPrimitive> QuadraticSurd<T>
+    where
+        for<'r> &'r T: RefNum<T>,
+    {
+        pub fn to_complex64(&self) -> Option<Complex64> {
+            if self.r < T::zero() {
+                let c = self.c.to_f64()?;
+                let re = self.a.to_f64()? / c;
+                let im = self.b.to_f64()? * self.r.to_f64()?.abs().sqrt() / c;
+                Some(Complex64::new(re as f64, im as f64))
+            } else {
+                let re = self.to_f64()?;
+                Some(Complex64::new(re as f64, 0f64))
+            }
+        }
+
+        pub fn to_complex32(&self) -> Option<Complex32> {
+            let complex = self.to_complex64()?;
+            Some(Complex32::new(complex.re.to_f32()?, complex.im.to_f32()?))
+        }
     }
 }
 
@@ -998,7 +1167,12 @@ where
         }
 
         match target.numer().checked_mul(target.denom()) {
-            Some(new_r) => Ok(QuadraticSurd::new(T::zero(), T::one(), target.denom().clone(), new_r)),
+            Some(new_r) => Ok(QuadraticSurd::new(
+                T::zero(),
+                T::one(),
+                target.denom().clone(),
+                new_r,
+            )),
             None => Err(FromSqrtError::Overflow),
         }
     }
@@ -1059,6 +1233,7 @@ where
 #[cfg(test)]
 mod tests {
     use super::*;
+    use core::convert::TryFrom;
 
     pub const PHI: QuadraticSurd<i32> = QuadraticSurd::new_raw(1, 1, 2, 5); // 1.618
     pub const PHI_R: QuadraticSurd<i32> = QuadraticSurd::new_raw(-1, 1, 2, 5); // 0.618
@@ -1069,14 +1244,17 @@ mod tests {
     pub const PHI45: QuadraticSurd<i32> = QuadraticSurd::new_raw(3, 1, 6, 45); // non-reduced version of PHI
     pub const PHI45_R: QuadraticSurd<i32> = QuadraticSurd::new_raw(-3, 1, 6, 45); // non-reduced version of PHI_R
 
+    pub const SQ5: QuadraticSurd<i32> = QuadraticSurd::new_raw(0, 1, 1, 5);
+    pub const N_SQ5: QuadraticSurd<i32> = QuadraticSurd::new_raw(0, -1, 1, 5);
+
     #[test]
-    fn from_rational_test() {
+    fn from_rationals_test() {
         assert_eq!(
-            QuadraticSurd::from_rational(Ratio::new(1, 2), Ratio::new(1, 2), Ratio::from(5)),
+            QuadraticSurd::from_rationals(Ratio::new(1, 2), Ratio::new(1, 2), Ratio::from(5)),
             PHI
         );
         assert_eq!(
-            QuadraticSurd::from_rational(Ratio::new(-1, 2), Ratio::new(1, 2), Ratio::from(5)),
+            QuadraticSurd::from_rationals(Ratio::new(-1, 2), Ratio::new(1, 2), Ratio::from(5)),
             PHI_R
         );
     }
@@ -1105,36 +1283,16 @@ mod tests {
             QuadraticSurd::new_raw(0, 1, 1, 5)
         );
         assert!(matches!(QuadraticSurd::from_sqrt(PHI), Err(_)));
-        assert!(matches!(QuadraticSurd::from_sqrt(PHI * PHI), Ok(PHI)));
+        assert!(matches!(QuadraticSurd::from_sqrt(PHI * PHI), Ok(v) if v == PHI));
 
         assert!(matches!(
             QuadraticSurd::from_equation(Ratio::from(1), Ratio::from(-1), Ratio::from(-1)),
-            Some(PHI)
+            Some(v) if v == PHI
         ));
     }
 
     #[test]
-    fn arithmic_test() {
-        let sq5 = QuadraticSurd::from_sqrt(5).unwrap();
-        assert_eq!(-sq5, QuadraticSurd::new_raw(0, -1, 1, 5));
-
-        // add
-        assert_eq!(PHI + PHI_R, sq5);
-        assert_eq!(PHI45 + PHI_R, sq5);
-        assert_eq!(PHI + PHI45_R, sq5);
-        assert_eq!(N_PHI + N_PHI_R, -sq5);
-        assert!((PHI + N_PHI).is_zero());
-        assert!((PHI + N_PHI_R).is_one());
-
-        // sub
-        assert!((PHI - PHI).is_zero());
-        assert!((PHI - PHI45).is_zero());
-        assert!((PHI - PHI_R).is_one());
-        assert!((PHI - PHI45_R).is_one());
-        assert!((N_PHI_R - N_PHI).is_one());
-        assert_eq!(PHI - N_PHI_R, sq5);
-        assert_eq!(N_PHI - PHI_R, -sq5);
-
+    fn property_test() {
         // recip
         assert_eq!(PHI.recip(), PHI_R);
         assert_eq!(N_PHI.recip(), N_PHI_R);
@@ -1143,7 +1301,47 @@ mod tests {
         assert_eq!(PHI.conj(), N_PHI_R);
         assert_eq!(PHI_R.conj(), N_PHI);
 
+        // is_pure
+        assert_eq!(PHI.is_pure(), false);
+        assert_eq!(SQ5.is_pure(), true);
+    }
+
+    #[test]
+    fn arithmic_test() {
+        // add
+        assert_eq!(PHI_R + 1, PHI);
+        assert_eq!(PHI45_R + 1, PHI45);
+        assert_eq!(N_PHI + 1, N_PHI_R);
+        assert_eq!(PHI_R + Ratio::one(), PHI);
+        assert_eq!(PHI45_R + Ratio::one(), PHI45);
+        assert_eq!(N_PHI + Ratio::one(), N_PHI_R);
+        assert_eq!(PHI + PHI_R, SQ5);
+        assert_eq!(PHI45 + PHI_R, SQ5);
+        assert_eq!(PHI + PHI45_R, SQ5);
+        assert_eq!(N_PHI + N_PHI_R, N_SQ5);
+        assert!((PHI + N_PHI).is_zero());
+        assert!((PHI + N_PHI_R).is_one());
+
+        // sub
+        assert_eq!(PHI - 1, PHI_R);
+        assert_eq!(PHI45 - 1, PHI45_R);
+        assert_eq!(N_PHI_R - 1, N_PHI);
+        assert_eq!(PHI - Ratio::one(), PHI_R);
+        assert_eq!(PHI45 - Ratio::one(), PHI45_R);
+        assert_eq!(N_PHI_R - Ratio::one(), N_PHI);
+        assert!((PHI - PHI).is_zero());
+        assert!((PHI - PHI45).is_zero());
+        assert!((PHI - PHI_R).is_one());
+        assert!((PHI - PHI45_R).is_one());
+        assert!((N_PHI_R - N_PHI).is_one());
+        assert_eq!(PHI - N_PHI_R, SQ5);
+        assert_eq!(N_PHI - PHI_R, -SQ5);
+
         // mul
+        assert_eq!(PHI * 1, PHI);
+        assert_eq!(PHI * -1, N_PHI);
+        assert_eq!(PHI * Ratio::one(), PHI);
+        assert_eq!(PHI * -Ratio::one(), N_PHI);
         assert!((PHI * PHI_R).is_one());
         assert!((PHI45 * PHI_R).is_one());
         assert!((PHI * PHI45_R).is_one());
@@ -1152,12 +1350,34 @@ mod tests {
         assert_eq!(N_PHI * N_PHI, PHI_SQ);
 
         // div
+        assert_eq!(PHI / 1, PHI);
+        assert_eq!(PHI / -1, N_PHI);
+        assert_eq!(PHI / Ratio::one(), PHI);
+        assert_eq!(PHI / -Ratio::one(), N_PHI);
         assert!((PHI / PHI).is_one());
         assert!((PHI45 / PHI).is_one());
         assert!((PHI / PHI45).is_one());
         assert!((N_PHI / N_PHI).is_one());
         assert_eq!(PHI / PHI_R, PHI_SQ);
         assert_eq!(N_PHI / N_PHI_R, PHI_SQ);
+
+        // associativity test
+        let three_half = Ratio::new(3, 2);
+        assert_eq!(PHI + 5 - PHI, QuadraticSurd::from(5));
+        assert_eq!(PHI + three_half - PHI, QuadraticSurd::from(three_half));
+        assert_eq!(PHI + SQ5 - PHI, SQ5);
+        assert_eq!(PHI + 5 - PHI45, QuadraticSurd::from(5));
+        assert_eq!(PHI + three_half - PHI45, QuadraticSurd::from(three_half));
+        assert_eq!(PHI + SQ5 - PHI45, SQ5);
+        assert_eq!(PHI * 5 / PHI45, QuadraticSurd::from(5));
+        assert_eq!(PHI * three_half / PHI45, QuadraticSurd::from(three_half));
+        assert_eq!(PHI * SQ5 / PHI45, SQ5);
+
+        // mixed
+        assert_eq!(PHI * 2 - 1, SQ5);
+        assert_eq!(PHI_R * 2 + 1, SQ5);
+        assert_eq!(PHI / Ratio::new(1, 2) - 1, SQ5);
+        assert_eq!((PHI - Ratio::new(1, 2)) * 2, SQ5);
     }
 
     #[test]
@@ -1184,11 +1404,13 @@ mod tests {
         let cf_10_sq2_7 = ContinuedFraction::<u32>::new(vec![1i32, 4], vec![2], false);
         let surd_10_sq2_7 = QuadraticSurd::new(10, -1, 7, 2);
         assert_eq!(QuadraticSurd::from(cf_10_sq2_7.clone()), surd_10_sq2_7);
-        assert_eq!(ContinuedFraction::try_from(surd_10_sq2_7).unwrap(), cf_10_sq2_7);
+        assert_eq!(
+            ContinuedFraction::try_from(surd_10_sq2_7).unwrap(),
+            cf_10_sq2_7
+        );
     }
 }
 
-
 #[cfg(test)]
 mod complex_tests {
     use super::*;
@@ -1196,9 +1418,19 @@ mod complex_tests {
     #[cfg(not(feature = "complex"))]
     #[test]
     fn from_sqrt_test() {
-        assert_eq!(QuadraticSurd::from_sqrt(-2).unwrap_err(), FromSqrtError::Complex);
-        assert_eq!(QuadraticSurd::from_sqrt(Ratio::new(-1, 2)).unwrap_err(), FromSqrtError::Complex);
-        assert_eq!(QuadraticSurd::from_sqrt(QuadraticSurd::from(-1)).unwrap_err(), FromSqrtError::Complex);
+        assert_eq!(QuadraticSurd::from_sqrt(5).unwrap(), QuadraticSurd::new_raw(0, 1, 1, 5));
+        assert_eq!(
+            QuadraticSurd::from_sqrt(-2).unwrap_err(),
+            FromSqrtError::Complex
+        );
+        assert_eq!(
+            QuadraticSurd::from_sqrt(Ratio::new(-1, 2)).unwrap_err(),
+            FromSqrtError::Complex
+        );
+        assert_eq!(
+            QuadraticSurd::from_sqrt(QuadraticSurd::from(-1)).unwrap_err(),
+            FromSqrtError::Complex
+        );
     }
 
     #[cfg(feature = "complex")]
